import { csv } from 'd3';
import {
  select,
  json,
  pie,
  arc,
  scaleOrdinal,
  format,
} from 'd3';

const sectionColors = [
  'green',
  'brown',
  'Pink',
  'skyblue',
  'Grey',
  'violet',
  'orange',
  'wheat',
  'lightgreen',
  'aqua',
]; //Different colors for representation
//dataset Url made raw from github
const csvUrl =
  'https://gist.githubusercontent.com/Mahesh-Polisetty/10961f786763719f1dd93e35dad35bf9/raw/b59d79e079a19150f06d00c763a46be071689a8c/piechart_carsales.csv';
//fetching data from dataset
const fetchData = async () => {
  const data = await csv(csvUrl);
  return data;
};

const svg = select('svg');
const width = +svg.attr('width');
const height = +svg.attr('height');

const xAxisTickFormat = (number) => format('.4~g')(number);

const render = (data, total) => {
  const pieData = pie().value((d) => d.CarSales)(data);
  const colors = scaleOrdinal()
    .domain(data)
    .range(sectionColors);

  const arcGenerator = d3
    .arc()
    .innerRadius(0)
    .outerRadius(250);

  //arc
  const segments = arc()
    .innerRadius(0)
    .outerRadius(175)
    .padAngle(0)
    .padRadius(0);

  const sections = svg
    .append('g')
    .attr('transform', `translate(250,250)`)
    .selectAll('path')
    .data(pieData);

  sections
    .enter()
    .append('path')
    .attr('d', segments)
    .attr('fill', (d) => colors(d.data.CarSales));
  //section label
  sections
    .enter()
    .append('text')
    .text(
      (d) => xAxisTickFormat(d.data.CarSales / 1000) + 'K',
    )
    .attr('transform', function (d) {
      return 'translate(' + arcGenerator.centroid(d) + ')';
    })
    .style('text-anchor', 'middle')
    .style('fill', 'black')
    .style('font-size', 15);
  //legends
  const legends = svg
    .append('g')
    .attr('transform', 'translate(500,100)')
    .selectAll('.legends')
    .data(pieData);

  const legend = legends
    .enter()
    .append('g')
    .classed('.legends', true)
    .attr('transform', (d, i) => {
      return `translate(0,${(i + 1) * 30})`;
    });

  legend
    .append('rect')
    .attr('width', 20)
    .attr('height', 20)
    .attr('fill', (d) => colors(d.data.CarSales));
  legend
    .append('text')
    .attr('x', 25)
    .attr('y', 15)
    .attr('class', 'legend_text')
    .text((d) => d.data.State);
  legend
    .append('text')
    .attr('x', 250)
    .attr('y', 15)
    .attr('class', 'legend_value')
    .text(
      (d) => xAxisTickFormat(d.data.CarSales / 1000) + 'K',
    );
};

const run = async () => {
  let data = await fetchData();
  let total = 0;
  data.forEach((d) => {
    d.CarSales = +d.CarSales;
    total += d.CarSales;
  });
  render(data, total);
};
run();
